@@ -1,6 +1,6 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 
 use std::collections::{HashMap, HashSet};
 
@@ -15,7 +15,10 @@ pub struct Reference {
 }
 
 impl Reference {
-    pub fn new<T>(reference_type: T, target_node: NodeId) -> Reference where T: Into<NodeId> {
+    pub fn new<T>(reference_type: T, target_node: NodeId) -> Reference
+    where
+        T: Into<NodeId>,
+    {
         Reference {
             reference_type: reference_type.into(),
             target_node,
@@ -50,8 +53,12 @@ impl Default for References {
 
 impl References {
     /// Inserts a single reference into the map.
-    pub fn insert<T>(&mut self, source_node: &NodeId, references: &[(&NodeId, &T, ReferenceDirection)])
-        where T: Into<NodeId> + Clone
+    pub fn insert<T>(
+        &mut self,
+        source_node: &NodeId,
+        references: &[(&NodeId, &T, ReferenceDirection)],
+    ) where
+        T: Into<NodeId> + Clone,
     {
         references.iter().for_each(|r| {
             // An inverse reference will flip the nodes around
@@ -73,38 +80,62 @@ impl References {
         } else if self.references_map.contains_key(node_id) {
             debug!("Node {} is a key in references_from_map", node_id);
             true
-        } else if self.references_map.iter().find(|(k, v)| {
-            if let Some(r) = v.iter().find(|r| r.target_node == *node_id) {
-                debug!("Node {} is a value in references_from_map[{}, reference = {:?}", node_id, k, r);
-                true
-            } else {
-                false
-            }
-        }).is_some() {
+        } else if self
+            .references_map
+            .iter()
+            .find(|(k, v)| {
+                if let Some(r) = v.iter().find(|r| r.target_node == *node_id) {
+                    debug!(
+                        "Node {} is a value in references_from_map[{}, reference = {:?}",
+                        node_id, k, r
+                    );
+                    true
+                } else {
+                    false
+                }
+            })
+            .is_some()
+        {
             true
-        } else if self.referenced_by_map.iter().find(|(k, v)| {
-            if v.contains(node_id) {
-                debug!("Node {} is a value in referenced_by_map, key {}", node_id, k);
-                true
-            } else {
-                false
-            }
-        }).is_some() {
+        } else if self
+            .referenced_by_map
+            .iter()
+            .find(|(k, v)| {
+                if v.contains(node_id) {
+                    debug!(
+                        "Node {} is a value in referenced_by_map, key {}",
+                        node_id, k
+                    );
+                    true
+                } else {
+                    false
+                }
+            })
+            .is_some()
+        {
             true
         } else {
             false
         }
     }
 
-    pub fn insert_reference<T>(&mut self, source_node: &NodeId, target_node: &NodeId, reference_type: &T)
-        where T: Into<NodeId> + Clone
+    pub fn insert_reference<T>(
+        &mut self,
+        source_node: &NodeId,
+        target_node: &NodeId,
+        reference_type: &T,
+    ) where
+        T: Into<NodeId> + Clone,
     {
         if source_node == target_node {
-            panic!("Node id from == node id to {}, self reference is not allowed", source_node);
+            panic!(
+                "Node id from == node id to {}, self reference is not allowed",
+                source_node
+            );
         }
 
         let reference_type: NodeId = reference_type.clone().into();
-        let reference = Reference::new(reference_type.clone(), target_node.clone());
+        let reference = Reference::new(reference_type, target_node.clone());
 
         if let Some(ref mut references) = self.references_map.get_mut(source_node) {
             // Duplicates are possible from the machine generated code, so skip dupes
@@ -125,40 +156,46 @@ impl References {
         } else {
             let mut lookup_set = HashSet::new();
             lookup_set.insert(source_node.clone());
-            self.referenced_by_map.insert(target_node.clone(), lookup_set);
+            self.referenced_by_map
+                .insert(target_node.clone(), lookup_set);
         }
     }
 
     /// Inserts references into the map.
     pub fn insert_references<T>(&mut self, references: &[(&NodeId, &NodeId, &T)])
-        where T: Into<NodeId> + Clone
+    where
+        T: Into<NodeId> + Clone,
     {
         references.iter().for_each(|r| {
             self.insert_reference(r.0, r.1, r.2);
         });
     }
 
-    fn remove_node_from_referenced_nodes(&mut self, nodes_to_check: HashSet<NodeId>, node_to_remove: &NodeId) {
+    fn remove_node_from_referenced_nodes(
+        &mut self,
+        nodes_to_check: HashSet<NodeId>,
+        node_to_remove: &NodeId,
+    ) {
         nodes_to_check.into_iter().for_each(|node_to_check| {
             // Removes any references that refer from the node to check back to the node to remove
-            let remove_entry = if let Some(ref mut references) = self.references_map.get_mut(&node_to_check) {
-                references.retain(|r| {
-                    r.target_node != *node_to_remove
-                });
-                references.is_empty()
-            } else {
-                false
-            };
+            let remove_entry =
+                if let Some(ref mut references) = self.references_map.get_mut(&node_to_check) {
+                    references.retain(|r| r.target_node != *node_to_remove);
+                    references.is_empty()
+                } else {
+                    false
+                };
             if remove_entry {
                 self.references_map.remove(&node_to_check);
             }
             // Remove lookup that refer from the node to check back to the node to remove
-            let remove_lookup_map = if let Some(ref mut lookup_map) = self.referenced_by_map.get_mut(&node_to_check) {
-                lookup_map.remove(node_to_remove);
-                lookup_map.is_empty()
-            } else {
-                false
-            };
+            let remove_lookup_map =
+                if let Some(ref mut lookup_map) = self.referenced_by_map.get_mut(&node_to_check) {
+                    lookup_map.remove(node_to_remove);
+                    lookup_map.is_empty()
+                } else {
+                    false
+                };
             if remove_lookup_map {
                 self.referenced_by_map.remove(&node_to_check);
             }
@@ -167,7 +204,15 @@ impl References {
 
     /// Deletes a matching references between one node and the target node of the specified
     /// reference type. The function returns true if the reference was found and deleted.
-    pub fn delete_reference<T>(&mut self, source_node: &NodeId, target_node: &NodeId, reference_type: T) -> bool where T: Into<NodeId> {
+    pub fn delete_reference<T>(
+        &mut self,
+        source_node: &NodeId,
+        target_node: &NodeId,
+        reference_type: T,
+    ) -> bool
+    where
+        T: Into<NodeId>,
+    {
         let reference_type = reference_type.into();
 
         let mut deleted = false;
@@ -175,7 +220,10 @@ impl References {
         // Remove the source node reference
         if let Some(references) = self.references_map.get_mut(source_node) {
             // Make a set of all the nodes that this node references
-            let other_nodes_before = references.iter().map(|r| r.target_node.clone()).collect::<HashSet<NodeId>>();
+            let other_nodes_before = references
+                .iter()
+                .map(|r| r.target_node.clone())
+                .collect::<HashSet<NodeId>>();
             // Delete a reference
             references.retain(|r| {
                 if r.reference_type == reference_type && r.target_node == *target_node {
@@ -190,11 +238,17 @@ impl References {
             }
 
             // Make a set of all nodes that this node references (after removal)
-            let other_nodes_after = references.iter().map(|r| r.target_node.clone()).collect::<HashSet<NodeId>>();
+            let other_nodes_after = references
+                .iter()
+                .map(|r| r.target_node.clone())
+                .collect::<HashSet<NodeId>>();
 
             // If nodes are no longer referenced, then the ones that were removed must also have their
             // references changed.
-            let difference = other_nodes_before.difference(&other_nodes_after).cloned().collect::<HashSet<NodeId>>();
+            let difference = other_nodes_before
+                .difference(&other_nodes_after)
+                .cloned()
+                .collect::<HashSet<NodeId>>();
             if !difference.is_empty() {
                 self.remove_node_from_referenced_nodes(difference, source_node);
             }
@@ -210,7 +264,10 @@ impl References {
     pub fn delete_node_references(&mut self, source_node: &NodeId) -> bool {
         let deleted_references = if let Some(references) = self.references_map.remove(source_node) {
             // Deleted every reference from the node, and clean up the reverse lookup map
-            let nodes_referenced = references.iter().map(|r| r.target_node.clone()).collect::<HashSet<NodeId>>();
+            let nodes_referenced = references
+                .iter()
+                .map(|r| r.target_node.clone())
+                .collect::<HashSet<NodeId>>();
             self.remove_node_from_referenced_nodes(nodes_referenced, source_node);
             true
         } else {
@@ -228,8 +285,16 @@ impl References {
     }
 
     /// Test if a reference relationship exists between one node and another node
-    pub fn has_reference<T>(&self, source_node: &NodeId, target_node: &NodeId, reference_type: T) -> bool where T: Into<NodeId> {
-        if let Some(references) = self.references_map.get(&source_node) {
+    pub fn has_reference<T>(
+        &self,
+        source_node: &NodeId,
+        target_node: &NodeId,
+        reference_type: T,
+    ) -> bool
+    where
+        T: Into<NodeId>,
+    {
+        if let Some(references) = self.references_map.get(source_node) {
             let reference = Reference::new(reference_type.into(), target_node.clone());
             references.contains(&reference)
         } else {
@@ -238,8 +303,15 @@ impl References {
     }
 
     /// Finds forward references from the node
-    pub fn find_references<T>(&self, source_node: &NodeId, reference_filter: Option<(T, bool)>) -> Option<Vec<Reference>> where T: Into<NodeId> + Clone {
-        if let Some(ref node_references) = self.references_map.get(source_node) {
+    pub fn find_references<T>(
+        &self,
+        source_node: &NodeId,
+        reference_filter: Option<(T, bool)>,
+    ) -> Option<Vec<Reference>>
+    where
+        T: Into<NodeId> + Clone,
+    {
+        if let Some(node_references) = self.references_map.get(source_node) {
             let result = self.filter_references_by_type(node_references, &reference_filter);
             if result.is_empty() {
                 None
@@ -254,22 +326,29 @@ impl References {
     /// Returns inverse references for the target node, i.e if there are references where
     /// `Reference.target_node` matches the supplied target node then return references
     /// where `Reference.target_node` is the source node.
-    pub fn find_inverse_references<T>(&self, target_node: &NodeId, reference_filter: Option<(T, bool)>) -> Option<Vec<Reference>> where T: Into<NodeId> + Clone {
+    pub fn find_inverse_references<T>(
+        &self,
+        target_node: &NodeId,
+        reference_filter: Option<(T, bool)>,
+    ) -> Option<Vec<Reference>>
+    where
+        T: Into<NodeId> + Clone,
+    {
         if let Some(lookup_map) = self.referenced_by_map.get(target_node) {
             // Iterate all nodes that reference this node, collecting their references
             let mut result = Vec::with_capacity(16);
             lookup_map.iter().for_each(|source_node| {
                 if let Some(references) = self.references_map.get(source_node) {
-                    let references = references.iter()
+                    let references = references
+                        .iter()
                         .filter(|r| r.target_node == *target_node)
-                        .map(|r| {
-                            Reference {
-                                reference_type: r.reference_type.clone(),
-                                target_node: source_node.clone(),
-                            }
+                        .map(|r| Reference {
+                            reference_type: r.reference_type.clone(),
+                            target_node: source_node.clone(),
                         })
                         .collect::<Vec<Reference>>();
-                    let mut references = self.filter_references_by_type(&references, &reference_filter);
+                    let mut references =
+                        self.filter_references_by_type(&references, &reference_filter);
                     if !references.is_empty() {
                         result.append(&mut references);
                     }
@@ -285,13 +364,27 @@ impl References {
         }
     }
 
-    fn filter_references_by_type<T>(&self, references: &Vec<Reference>, reference_filter: &Option<(T, bool)>) -> Vec<Reference> where T: Into<NodeId> + Clone {
+    fn filter_references_by_type<T>(
+        &self,
+        references: &[Reference],
+        reference_filter: &Option<(T, bool)>,
+    ) -> Vec<Reference>
+    where
+        T: Into<NodeId> + Clone,
+    {
         match reference_filter {
-            None => references.clone(),
+            None => references.to_owned(),
             Some((reference_type_id, include_subtypes)) => {
                 let reference_type_id = reference_type_id.clone().into();
-                references.iter()
-                    .filter(|r| self.reference_type_matches(&reference_type_id, &r.reference_type, *include_subtypes))
+                references
+                    .iter()
+                    .filter(|r| {
+                        self.reference_type_matches(
+                            &reference_type_id,
+                            &r.reference_type,
+                            *include_subtypes,
+                        )
+                    })
                     .cloned()
                     .collect::<Vec<Reference>>()
             }
@@ -301,8 +394,14 @@ impl References {
     /// Find references optionally to and/or from the specified node id. The browse direction
     /// indicates the desired direction, or both. The reference filter indicates if only references
     /// of a certain type (including sub types) should be fetched.
-    pub fn find_references_by_direction<T>(&self, node: &NodeId, browse_direction: BrowseDirection, reference_filter: Option<(T, bool)>) -> (Vec<Reference>, usize)
-        where T: Into<NodeId> + Clone
+    pub fn find_references_by_direction<T>(
+        &self,
+        node: &NodeId,
+        browse_direction: BrowseDirection,
+        reference_filter: Option<(T, bool)>,
+    ) -> (Vec<Reference>, usize)
+    where
+        T: Into<NodeId> + Clone,
     {
         let mut references = Vec::new();
         let inverse_ref_idx: usize;
@@ -315,17 +414,26 @@ impl References {
             }
             BrowseDirection::Inverse => {
                 inverse_ref_idx = 0;
-                if let Some(mut inverse_references) = self.find_inverse_references(node, reference_filter) {
+                if let Some(mut inverse_references) =
+                    self.find_inverse_references(node, reference_filter)
+                {
                     references.append(&mut inverse_references);
                 }
             }
             BrowseDirection::Both => {
-                let reference_filter: Option<(NodeId, bool)> = reference_filter.map(|(reference_type, include_subtypes)| (reference_type.into(), include_subtypes));
-                if let Some(mut forward_references) = self.find_references(node, reference_filter.clone()) {
+                let reference_filter: Option<(NodeId, bool)> =
+                    reference_filter.map(|(reference_type, include_subtypes)| {
+                        (reference_type.into(), include_subtypes)
+                    });
+                if let Some(mut forward_references) =
+                    self.find_references(node, reference_filter.clone())
+                {
                     references.append(&mut forward_references);
                 }
                 inverse_ref_idx = references.len();
-                if let Some(mut inverse_references) = self.find_inverse_references(node, reference_filter) {
+                if let Some(mut inverse_references) =
+                    self.find_inverse_references(node, reference_filter)
+                {
                     references.append(&mut inverse_references);
                 }
             }
@@ -340,7 +448,12 @@ impl References {
     /// Test if a reference type matches another reference type which is potentially a subtype.
     /// If `include_subtypes` is set to true, the function will test if the subttype
     /// for a match.
-    pub fn reference_type_matches(&self, ref_type: &NodeId, ref_subtype: &NodeId, include_subtypes: bool) -> bool {
+    pub fn reference_type_matches(
+        &self,
+        ref_type: &NodeId,
+        ref_subtype: &NodeId,
+        include_subtypes: bool,
+    ) -> bool {
         if ref_type == ref_subtype {
             true
         } else if include_subtypes {
@@ -357,7 +470,8 @@ impl References {
                     found = true;
                     break;
                 } else if let Some(references) = self.references_map.get(&current) {
-                    let mut subtypes = references.iter()
+                    let mut subtypes = references
+                        .iter()
                         .filter(|r| r.reference_type == has_subtype)
                         .map(|r| r.target_node.clone())
                         .collect::<Vec<NodeId>>();
@@ -375,17 +489,14 @@ impl References {
     }
 
     pub fn get_type_id(&self, node: &NodeId) -> Option<NodeId> {
-        if let Some(references) = self.references_map.get(&node) {
+        if let Some(references) = self.references_map.get(node) {
             let has_type_definition_id = ReferenceTypeId::HasTypeDefinition.into();
-            if let Some(reference) = references.iter().find(|r| {
-                r.reference_type == has_type_definition_id
-            }) {
-                Some(reference.target_node.clone())
-            } else {
-                None
-            }
+            references
+                .iter()
+                .find(|r| r.reference_type == has_type_definition_id)
+                .map(|reference| reference.target_node.clone())
         } else {
             None
         }
     }
-}
\ No newline at end of file
+}
