@@ -1,15 +1,12 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 
 //! Provides a browse continuation point type for tracking a browse operation initiated by a client.
 
 use std::sync::{Arc, Mutex};
 
-use opcua_types::{
-    ByteString, DateTimeUtc,
-    service_types::ReferenceDescription,
-};
+use opcua_types::{service_types::ReferenceDescription, ByteString, DateTimeUtc};
 
 use crate::prelude::AddressSpace;
 
@@ -28,4 +25,4 @@ impl BrowseContinuationPoint {
     pub fn is_valid_browse_continuation_point(&self, address_space: &AddressSpace) -> bool {
         self.address_space_last_modified >= address_space.last_modified()
     }
-}
\ No newline at end of file
+}
