@@ -1,29 +1,26 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 use std::{
-    collections::{HashSet, VecDeque},
+    collections::{HashMap, HashSet, VecDeque},
     sync::{
-        Arc, atomic::{AtomicI32, Ordering},
-        RwLock,
+        atomic::{AtomicI32, Ordering},
+        Arc, RwLock,
     },
 };
 
-use chrono::{self, Utc};
+use chrono::Utc;
 
-use opcua_core::comms::secure_channel::{Role, SecureChannel};
 use opcua_crypto::X509;
-use opcua_types::{
-    *, service_types::PublishRequest, status_code::StatusCode,
-};
+use opcua_types::{service_types::PublishRequest, status_code::StatusCode, *};
 
 use crate::{
     address_space::{AddressSpace, UserAccessLevel},
     continuation_point::BrowseContinuationPoint,
     diagnostics::ServerDiagnostics,
     identity_token::IdentityToken,
-    server::Server,
     session_diagnostics::SessionDiagnostics,
+    state::ServerState,
     subscriptions::subscription::TickReason,
     subscriptions::subscriptions::Subscriptions,
 };
@@ -53,11 +50,93 @@ pub enum ServerUserIdentityToken {
     Invalid(ExtensionObject),
 }
 
+pub struct SessionManager {
+    pub sessions: HashMap<NodeId, Arc<RwLock<Session>>>,
+    pub sessions_terminated: bool,
+}
+
+impl Default for SessionManager {
+    fn default() -> Self {
+        Self {
+            sessions: HashMap::new(),
+            sessions_terminated: false,
+        }
+    }
+}
+
+impl SessionManager {
+    pub fn len(&self) -> usize {
+        self.sessions.len()
+    }
+
+    pub fn first(&self) -> Option<Arc<RwLock<Session>>> {
+        self.sessions.iter().next().map(|(_, s)| s.clone())
+    }
+
+    pub fn sessions_terminated(&self) -> bool {
+        self.sessions_terminated
+    }
+
+    /// Puts all sessions into a terminated state and clears the map
+    pub fn clear(&mut self) {
+        self.sessions.iter().for_each(|s| {
+            let mut session = trace_write_lock!(s.1);
+            session.set_terminated();
+        });
+        self.sessions.clear();
+    }
+
+    /// Find a session by its session id and return it.
+    pub fn find_session_by_id(&self, session_id: &NodeId) -> Option<Arc<RwLock<Session>>> {
+        self.sessions
+            .iter()
+            .find(|s| {
+                let session = trace_read_lock!(s.1);
+                session.session_id() == session_id
+            })
+            .map(|s| s.1)
+            .cloned()
+    }
+
+    /// Finds the session by its authentication token and returns it. The authentication token
+    /// can be renewed so  it is not used as a key.
+    pub fn find_session_by_token(
+        &self,
+        authentication_token: &NodeId,
+    ) -> Option<Arc<RwLock<Session>>> {
+        self.sessions
+            .iter()
+            .find(|s| {
+                let session = trace_read_lock!(s.1);
+                session.authentication_token() == authentication_token
+            })
+            .map(|s| s.1)
+            .cloned()
+    }
+
+    /// Register the session in the map so it can be searched on
+    pub fn register_session(&mut self, session: Arc<RwLock<Session>>) {
+        let session_id = {
+            let session = trace_read_lock!(session);
+            session.session_id().clone()
+        };
+        self.sessions.insert(session_id, session);
+    }
+
+    /// Deregisters a session from the map
+    pub fn deregister_session(
+        &mut self,
+        session: Arc<RwLock<Session>>,
+    ) -> Option<Arc<RwLock<Session>>> {
+        let session = trace_read_lock!(session);
+        let result = self.sessions.remove(session.session_id());
+        self.sessions_terminated = self.sessions.is_empty();
+        result
+    }
+}
 
 /// The Session is any state maintained between the client and server
 pub struct Session {
-    /// Subscriptions associated with the session
-    subscriptions: Subscriptions,
     /// The session identifier
     session_id: NodeId,
     /// Security policy
@@ -66,8 +145,6 @@ pub struct Session {
     client_certificate: Option<X509>,
     /// Authentication token for the session
     authentication_token: NodeId,
-    /// Secure channel state
-    secure_channel: Arc<RwLock<SecureChannel>>,
     /// Session nonce
     session_nonce: ByteString,
     /// Session name (supplied by client)
@@ -105,19 +182,21 @@ pub struct Session {
     can_modify_address_space: bool,
     /// Timestamp of the last service request to have happened (only counts service requests while there is a session)
     last_service_request_timestamp: DateTimeUtc,
+    /// Subscriptions associated with the session
+    subscriptions: Subscriptions,
 }
 
 impl Drop for Session {
     fn drop(&mut self) {
         info!("Session is being dropped");
-        let mut diagnostics = trace_write_lock_unwrap!(self.diagnostics);
+        let mut diagnostics = trace_write_lock!(self.diagnostics);
         diagnostics.on_destroy_session(self);
     }
 }
 
 impl Session {
     #[cfg(test)]
-    pub fn new_no_certificate_store(secure_channel: SecureChannel) -> Session {
+    pub fn new_no_certificate_store() -> Session {
         let max_browse_continuation_points = super::constants::MAX_BROWSE_CONTINUATION_POINTS;
         let session = Session {
             subscriptions: Subscriptions::new(100, PUBLISH_REQUEST_TIMEOUT),
@@ -129,7 +208,6 @@ impl Session {
             client_certificate: None,
             security_policy_uri: String::new(),
             authentication_token: NodeId::null(),
-            secure_channel: Arc::new(RwLock::new(secure_channel)),
             session_nonce: ByteString::null(),
             session_name: UAString::null(),
             session_timeout: 0f64,
@@ -145,24 +223,24 @@ impl Session {
             session_diagnostics: Arc::new(RwLock::new(SessionDiagnostics::default())),
             last_service_request_timestamp: Utc::now(),
         };
+
         {
-            let mut diagnostics = trace_write_lock_unwrap!(session.diagnostics);
+            let mut diagnostics = trace_write_lock!(session.diagnostics);
             diagnostics.on_create_session(&session);
         }
         session
     }
 
     /// Create a `Session` from a `Server`
-    pub fn new(server: &Server) -> Session {
+    pub fn new(server_state: Arc<RwLock<ServerState>>) -> Session {
         let max_browse_continuation_points = super::constants::MAX_BROWSE_CONTINUATION_POINTS;
 
-        let server_state = server.server_state();
-        let server_state = trace_read_lock_unwrap!(server_state);
+        let server_state = trace_read_lock!(server_state);
         let max_subscriptions = server_state.max_subscriptions;
         let diagnostics = server_state.diagnostics.clone();
-        let (decoding_limits, can_modify_address_space) = {
-            let config = trace_read_lock_unwrap!(server_state.config);
-            (config.decoding_limits(), config.limits.clients_can_modify_address_space)
+        let can_modify_address_space = {
+            let config = trace_read_lock!(server_state.config);
+            config.limits.clients_can_modify_address_space
         };
 
         let session = Session {
@@ -175,7 +253,6 @@ impl Session {
             client_certificate: None,
             security_policy_uri: String::new(),
             authentication_token: NodeId::null(),
-            secure_channel: Arc::new(RwLock::new(SecureChannel::new(server.certificate_store(), Role::Server, decoding_limits))),
             session_nonce: ByteString::null(),
             session_name: UAString::null(),
             session_timeout: 0f64,
@@ -192,24 +269,31 @@ impl Session {
             last_service_request_timestamp: Utc::now(),
         };
         {
-            let mut diagnostics = trace_write_lock_unwrap!(session.diagnostics);
+            let mut diagnostics = trace_write_lock!(session.diagnostics);
             diagnostics.on_create_session(&session);
         }
         session
     }
-    pub fn secure_channel(&self) -> Arc<RwLock<SecureChannel>> { self.secure_channel.clone() }
 
-    pub fn session_id(&self) -> &NodeId { &self.session_id }
+    pub fn session_id(&self) -> &NodeId {
+        &self.session_id
+    }
 
     pub fn set_activated(&mut self, activated: bool) {
         self.activated = activated;
     }
 
-    pub fn is_activated(&self) -> bool { self.activated }
+    pub fn is_activated(&self) -> bool {
+        self.activated
+    }
 
-    pub fn is_terminated(&self) -> bool { self.terminated }
+    pub fn is_terminated(&self) -> bool {
+        self.terminated
+    }
 
-    pub fn terminated_at(&self) -> DateTimeUtc { self.terminated_at.clone() }
+    pub fn terminated_at(&self) -> DateTimeUtc {
+        self.terminated_at
+    }
 
     pub fn set_terminated(&mut self) {
         info!("Session being set to terminated");
@@ -258,10 +342,13 @@ impl Session {
     }
 
     pub fn last_service_request_timestamp(&self) -> DateTimeUtc {
-        self.last_service_request_timestamp.clone()
+        self.last_service_request_timestamp
     }
 
-    pub fn set_last_service_request_timestamp(&mut self, last_service_request_timestamp: DateTimeUtc) {
+    pub fn set_last_service_request_timestamp(
+        &mut self,
+        last_service_request_timestamp: DateTimeUtc,
+    ) {
         self.last_service_request_timestamp = last_service_request_timestamp;
     }
 
@@ -289,7 +376,9 @@ impl Session {
         self.session_nonce = session_nonce;
     }
 
-    pub fn session_name(&self) -> &UAString { &self.session_name }
+    pub fn session_name(&self) -> &UAString {
+        &self.session_name
+    }
 
     pub fn set_session_name(&mut self, session_name: UAString) {
         self.session_name = session_name;
@@ -307,11 +396,23 @@ impl Session {
         &mut self.subscriptions
     }
 
-    pub(crate) fn enqueue_publish_request(&mut self, now: &DateTimeUtc, request_id: u32, request: PublishRequest, address_space: &AddressSpace) -> Result<(), StatusCode> {
-        self.subscriptions.enqueue_publish_request(now, request_id, request, address_space)
+    pub(crate) fn enqueue_publish_request(
+        &mut self,
+        now: &DateTimeUtc,
+        request_id: u32,
+        request: PublishRequest,
+        address_space: &AddressSpace,
+    ) -> Result<(), StatusCode> {
+        self.subscriptions
+            .enqueue_publish_request(now, request_id, request, address_space)
     }
 
-    pub(crate) fn tick_subscriptions(&mut self, now: &DateTimeUtc, address_space: &AddressSpace, reason: TickReason) -> Result<(), StatusCode> {
+    pub(crate) fn tick_subscriptions(
+        &mut self,
+        now: &DateTimeUtc,
+        address_space: &AddressSpace,
+        reason: TickReason,
+    ) -> Result<(), StatusCode> {
         self.subscriptions.tick(now, address_space, reason)
     }
 
@@ -329,25 +430,42 @@ impl Session {
         self.subscriptions.expire_stale_publish_requests(now);
     }
 
-    pub(crate) fn add_browse_continuation_point(&mut self, continuation_point: BrowseContinuationPoint) {
+    pub(crate) fn add_browse_continuation_point(
+        &mut self,
+        continuation_point: BrowseContinuationPoint,
+    ) {
         // Remove excess browse continuation points
         while self.browse_continuation_points.len() >= self.max_browse_continuation_points {
             let continuation_point = self.browse_continuation_points.pop_front();
-            debug!("Removing old continuation point {} to make way for new one", continuation_point.unwrap().id.as_base64());
+            debug!(
+                "Removing old continuation point {} to make way for new one",
+                continuation_point.unwrap().id.as_base64()
+            );
         }
-        self.browse_continuation_points.push_back(continuation_point);
+        self.browse_continuation_points
+            .push_back(continuation_point);
     }
 
     /// Finds and REMOVES a continuation point by id.
-    pub(crate) fn find_browse_continuation_point(&mut self, id: &ByteString) -> Option<BrowseContinuationPoint> {
-        if let Some(idx) = self.browse_continuation_points.iter().position(|continuation_point| continuation_point.id == *id) {
+    pub(crate) fn find_browse_continuation_point(
+        &mut self,
+        id: &ByteString,
+    ) -> Option<BrowseContinuationPoint> {
+        if let Some(idx) = self
+            .browse_continuation_points
+            .iter()
+            .position(|continuation_point| continuation_point.id == *id)
+        {
             self.browse_continuation_points.remove(idx)
         } else {
             None
         }
     }
 
-    pub(crate) fn remove_expired_browse_continuation_points(&mut self, address_space: &AddressSpace) {
+    pub(crate) fn remove_expired_browse_continuation_points(
+        &mut self,
+        address_space: &AddressSpace,
+    ) {
         self.browse_continuation_points.retain(|continuation_point| {
             let valid = continuation_point.is_valid_browse_continuation_point(address_space);
             if !valid {
@@ -360,11 +478,11 @@ impl Session {
     /// Remove all the specified continuation points by id
     pub(crate) fn remove_browse_continuation_points(&mut self, continuation_points: &[ByteString]) {
         // Turn the supplied slice into a set
-        let continuation_points_set: HashSet<ByteString> = continuation_points.iter().cloned().collect();
+        let continuation_points_set: HashSet<ByteString> =
+            continuation_points.iter().cloned().collect();
         // Now remove any continuation points that are part of that set
-        self.browse_continuation_points.retain(|continuation_point| {
-            !continuation_points_set.contains(&continuation_point.id)
-        });
+        self.browse_continuation_points
+            .retain(|continuation_point| !continuation_points_set.contains(&continuation_point.id));
     }
 
     pub(crate) fn can_modify_address_space(&self) -> bool {
@@ -376,7 +494,12 @@ impl Session {
         self.can_modify_address_space = can_modify_address_space;
     }
 
-    pub(crate) fn effective_user_access_level(&self, user_access_level: UserAccessLevel, _node_id: &NodeId, _attribute_id: AttributeId) -> UserAccessLevel {
+    pub(crate) fn effective_user_access_level(
+        &self,
+        user_access_level: UserAccessLevel,
+        _node_id: &NodeId,
+        _attribute_id: AttributeId,
+    ) -> UserAccessLevel {
         // TODO session could modify the user_access_level further here via user / groups
         user_access_level
     }
@@ -387,9 +510,7 @@ impl Session {
     pub fn client_user_id(&self) -> UAString {
         match self.user_identity {
             IdentityToken::None | IdentityToken::AnonymousIdentityToken(_) => UAString::null(),
-            IdentityToken::UserNameIdentityToken(ref token) => {
-                token.user_name.clone()
-            }
+            IdentityToken::UserNameIdentityToken(ref token) => token.user_name.clone(),
             IdentityToken::X509IdentityToken(ref token) => {
                 if let Ok(cert) = X509::from_byte_string(&token.certificate_data) {
                     UAString::from(cert.subject_name())
@@ -397,18 +518,10 @@ impl Session {
                     UAString::from("Invalid certificate")
                 }
             }
-            IdentityToken::Invalid(_) => {
-                UAString::from("invalid")
-            }
+            IdentityToken::Invalid(_) => UAString::from("invalid"),
         }
     }
 
-    /// Helper function to return the secure channel id as a string
-    pub fn secure_channel_id(&self) -> String {
-        let secure_channel = trace_read_lock_unwrap!(self.secure_channel);
-        format!("{}", secure_channel.secure_channel_id())
-    }
-
     pub fn is_session_terminated(&self) -> bool {
         self.terminate_session
     }
@@ -418,14 +531,14 @@ impl Session {
     }
 
     pub(crate) fn register_session(&self, address_space: Arc<RwLock<AddressSpace>>) {
-        let session_diagnostics = trace_read_lock_unwrap!(self.session_diagnostics);
-        let mut address_space = trace_write_lock_unwrap!(address_space);
+        let session_diagnostics = trace_read_lock!(self.session_diagnostics);
+        let mut address_space = trace_write_lock!(address_space);
         session_diagnostics.register_session(self, &mut address_space);
     }
 
     pub(crate) fn deregister_session(&self, address_space: Arc<RwLock<AddressSpace>>) {
-        let session_diagnostics = trace_read_lock_unwrap!(self.session_diagnostics);
-        let mut address_space = trace_write_lock_unwrap!(address_space);
+        let session_diagnostics = trace_read_lock!(self.session_diagnostics);
+        let mut address_space = trace_write_lock!(address_space);
         session_diagnostics.deregister_session(self, &mut address_space);
     }
 }
