@@ -1,12 +1,9 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 
-use opcua_types::{
-    status_code::StatusCode,
-    service_types::PublishRequest,
-};
 use opcua_core::supported_message::SupportedMessage;
+use opcua_types::{service_types::PublishRequest, status_code::StatusCode};
 
 /// The publish request entry preserves the request_id which is part of the chunk layer but clients
 /// are fickle about receiving responses from the same as the request. Normally this is easy because
@@ -28,8 +25,6 @@ pub struct PublishResponseEntry {
     pub response: SupportedMessage,
 }
 
-use time;
-
 /// This converts an OPC UA Duration into a time duration used for testing for interval elapsed
 fn duration_from_ms(d: f64) -> time::Duration {
     // Duration is a floating point number in millis so turn to microseconds for greater accuracy
@@ -37,6 +32,6 @@ fn duration_from_ms(d: f64) -> time::Duration {
     time::Duration::microseconds((d * 1000f64) as i64)
 }
 
-pub mod subscriptions;
-pub mod subscription;
 pub mod monitored_item;
+pub mod subscription;
+pub mod subscriptions;
