@@ -2,10 +2,7 @@ use chrono::Duration;
 
 use opcua_types::{Variant, WriteMask};
 
-use crate::{
-    address_space::AccessLevel,
-    services::attribute::AttributeService,
-};
+use crate::{address_space::AccessLevel, services::attribute::AttributeService};
 
 use super::*;
 
@@ -18,7 +15,11 @@ fn read_value(node_id: &NodeId, attribute_id: AttributeId) -> ReadValueId {
     }
 }
 
-fn read_value_range(node_id: &NodeId, attribute_id: AttributeId, index_range: UAString) -> ReadValueId {
+fn read_value_range(
+    node_id: &NodeId,
+    attribute_id: AttributeId,
+    index_range: UAString,
+) -> ReadValueId {
     ReadValueId {
         node_id: node_id.clone(),
         attribute_id: attribute_id as u32,
@@ -27,7 +28,11 @@ fn read_value_range(node_id: &NodeId, attribute_id: AttributeId, index_range: UA
     }
 }
 
-fn read_value_encoding(node_id: &NodeId, attribute_id: AttributeId, data_encoding: QualifiedName) -> ReadValueId {
+fn read_value_encoding(
+    node_id: &NodeId,
+    attribute_id: AttributeId,
+    data_encoding: QualifiedName,
+) -> ReadValueId {
     ReadValueId {
         node_id: node_id.clone(),
         attribute_id: attribute_id as u32,
@@ -38,20 +43,33 @@ fn read_value_encoding(node_id: &NodeId, attribute_id: AttributeId, data_encodin
 
 fn node_ids(address_space: Arc<RwLock<AddressSpace>>) -> Vec<NodeId> {
     let (_, node_ids) = add_many_vars_to_address_space(address_space.clone(), 10);
-    let mut address_space = trace_write_lock_unwrap!(address_space);
+    let mut address_space = trace_write_lock!(address_space);
     // Remove read access to [3] for a test below
     let node = address_space.find_node_mut(&node_ids[3]).unwrap();
-    let r = node.as_mut_node().set_attribute(AttributeId::UserAccessLevel, Variant::from(0u8));
+    let r = node
+        .as_mut_node()
+        .set_attribute(AttributeId::UserAccessLevel, Variant::from(0u8));
     assert!(r.is_ok());
     node_ids
 }
 
 fn do_attribute_service_test<F>(f: F)
-    where F: FnOnce(Arc<RwLock<ServerState>>, Arc<RwLock<Session>>, Arc<RwLock<AddressSpace>>, &AttributeService)
+where
+    F: FnOnce(
+        Arc<RwLock<ServerState>>,
+        Arc<RwLock<Session>>,
+        Arc<RwLock<AddressSpace>>,
+        &AttributeService,
+    ),
 {
     // Set up some nodes
     let st = ServiceTest::new();
-    f(st.server_state.clone(), st.session.clone(), st.address_space.clone(), &AttributeService::new())
+    f(
+        st.server_state.clone(),
+        st.session.clone(),
+        st.address_space.clone(),
+        &AttributeService::new(),
+    )
 }
 
 #[test]
@@ -103,25 +121,40 @@ fn read() {
             assert!(results[1].server_timestamp.is_none());
 
             // 3. a variable without the required attribute
-            assert_eq!(results[2].status.as_ref().unwrap(), &StatusCode::BadAttributeIdInvalid);
+            assert_eq!(
+                results[2].status.as_ref().unwrap(),
+                &StatusCode::BadAttributeIdInvalid
+            );
             assert!(results[2].source_timestamp.is_none());
             assert!(results[2].server_timestamp.is_none());
 
             // 4. a variable with no read access
-            assert_eq!(results[3].status.as_ref().unwrap(), &StatusCode::BadNotReadable);
+            assert_eq!(
+                results[3].status.as_ref().unwrap(),
+                &StatusCode::BadNotReadable
+            );
             assert!(results[3].source_timestamp.is_none());
             assert!(results[3].server_timestamp.is_none());
 
             // 5. Non existent
-            assert_eq!(results[4].status.as_ref().unwrap(), &StatusCode::BadNodeIdUnknown);
+            assert_eq!(
+                results[4].status.as_ref().unwrap(),
+                &StatusCode::BadNodeIdUnknown
+            );
             assert!(results[4].source_timestamp.is_none());
             assert!(results[4].server_timestamp.is_none());
 
             // 6. Index range on a non-value
-            assert_eq!(results[5].status.as_ref().unwrap(), &StatusCode::BadIndexRangeNoData);
+            assert_eq!(
+                results[5].status.as_ref().unwrap(),
+                &StatusCode::BadIndexRangeNoData
+            );
 
             // 7. Invalid encoding
-            assert_eq!(results[6].status.as_ref().unwrap(), &StatusCode::BadDataEncodingInvalid);
+            assert_eq!(
+                results[6].status.as_ref().unwrap(),
+                &StatusCode::BadDataEncodingInvalid
+            );
         }
 
         // OTHER POTENTIAL TESTS
@@ -142,9 +175,7 @@ fn read_invalid_timestamps() {
         let node_ids = node_ids(address_space.clone());
 
         // Read a non existent variable
-        let nodes_to_read = vec![
-            read_value(&node_ids[0], AttributeId::Value),
-        ];
+        let nodes_to_read = vec![read_value(&node_ids[0], AttributeId::Value)];
         let request = ReadRequest {
             request_header: make_request_header(),
             max_age: 0f64,
@@ -155,7 +186,10 @@ fn read_invalid_timestamps() {
         let response = ats.read(server_state, session, address_space, &request);
         let response = supported_message_as!(response, ServiceFault);
 
-        assert_eq!(response.response_header.service_result, StatusCode::BadTimestampsToReturnInvalid);
+        assert_eq!(
+            response.response_header.service_result,
+            StatusCode::BadTimestampsToReturnInvalid
+        );
     });
 }
 
@@ -168,8 +202,15 @@ fn write_value(node_id: &NodeId, attribute_id: AttributeId, value: DataValue) ->
     }
 }
 
-fn write_value_index_range<V>(node_id: &NodeId, attribute_id: AttributeId, index_range: V, value: DataValue) -> WriteValue
-    where V: Into<UAString> {
+fn write_value_index_range<V>(
+    node_id: &NodeId,
+    attribute_id: AttributeId,
+    index_range: V,
+    value: DataValue,
+) -> WriteValue
+where
+    V: Into<UAString>,
+{
     WriteValue {
         node_id: node_id.clone(),
         attribute_id: attribute_id as u32,
@@ -179,7 +220,13 @@ fn write_value_index_range<V>(node_id: &NodeId, attribute_id: AttributeId, index
 }
 
 // Boiler plate helper makes a request and grabs a response
-fn write_request(server_state: Arc<RwLock<ServerState>>, session: Arc<RwLock<Session>>, address_space: Arc<RwLock<AddressSpace>>, ats: &AttributeService, nodes_to_write: Vec<WriteValue>) -> WriteResponse {
+fn write_request(
+    server_state: Arc<RwLock<ServerState>>,
+    session: Arc<RwLock<Session>>,
+    address_space: Arc<RwLock<AddressSpace>>,
+    ats: &AttributeService,
+    nodes_to_write: Vec<WriteValue>,
+) -> WriteResponse {
     let request = WriteRequest {
         request_header: make_request_header(),
         nodes_to_write: Some(nodes_to_write),
@@ -190,11 +237,19 @@ fn write_request(server_state: Arc<RwLock<ServerState>>, session: Arc<RwLock<Ses
 }
 
 // Boiler plate helper to get the node's value for verification
-fn validate_variable_value<F>(address_space: Arc<RwLock<AddressSpace>>, node_id: &NodeId, f: F) where F: FnOnce(&Variant) {
-    let address_space = trace_read_lock_unwrap!(address_space);
+fn validate_variable_value<F>(address_space: Arc<RwLock<AddressSpace>>, node_id: &NodeId, f: F)
+where
+    F: FnOnce(&Variant),
+{
+    let address_space = trace_read_lock!(address_space);
     let node = address_space.find_node(&node_id).unwrap();
     if let NodeType::Variable(node) = node {
-        let value = node.value(TimestampsToReturn::Neither, NumericRange::None, &QualifiedName::null(), 0.);
+        let value = node.value(
+            TimestampsToReturn::Neither,
+            NumericRange::None,
+            &QualifiedName::null(),
+            0.,
+        );
         f(&value.value.unwrap());
     } else {
         panic!();
@@ -209,7 +264,7 @@ fn write() {
         // can see what happens when they are written to.
         let node_ids = {
             let (_, node_ids) = add_many_vars_to_address_space(address_space.clone(), 10);
-            let mut address_space = trace_write_lock_unwrap!(address_space);
+            let mut address_space = trace_write_lock!(address_space);
             // set up nodes for the tests to be performed to each
             for (i, node_id) in node_ids.iter().enumerate() {
                 let node = address_space.find_node_mut(node_id).unwrap();
@@ -220,21 +275,38 @@ fn write() {
                     }
                     2 => {
                         // Remove write access to the value by setting access level to 0
-                        let _ = node.as_mut_node().set_attribute(AttributeId::UserAccessLevel, Variant::from(0u8)).unwrap();
+                        let _ = node
+                            .as_mut_node()
+                            .set_attribute(AttributeId::UserAccessLevel, Variant::from(0u8))
+                            .unwrap();
                     }
                     6 => {
                         node.as_mut_node().set_write_mask(WriteMask::ACCESS_LEVEL);
                     }
                     _ => {
                         // Write access
-                        let _ = node.as_mut_node().set_attribute(AttributeId::AccessLevel, Variant::from(AccessLevel::CURRENT_WRITE.bits())).unwrap();
-                        let _ = node.as_mut_node().set_attribute(AttributeId::UserAccessLevel, Variant::from(UserAccessLevel::CURRENT_WRITE.bits())).unwrap();
+                        let _ = node
+                            .as_mut_node()
+                            .set_attribute(
+                                AttributeId::AccessLevel,
+                                Variant::from(AccessLevel::CURRENT_WRITE.bits()),
+                            )
+                            .unwrap();
+                        let _ = node
+                            .as_mut_node()
+                            .set_attribute(
+                                AttributeId::UserAccessLevel,
+                                Variant::from(UserAccessLevel::CURRENT_WRITE.bits()),
+                            )
+                            .unwrap();
                     }
                 }
             }
 
             // change HasEncoding node with write access so response can be compared to HasChild which will be left alone
-            let node = address_space.find_node_mut(&ReferenceTypeId::HasEncoding.into()).unwrap();
+            let node = address_space
+                .find_node_mut(&ReferenceTypeId::HasEncoding.into())
+                .unwrap();
             node.as_mut_node().set_write_mask(WriteMask::IS_ABSTRACT);
 
             node_ids
@@ -246,26 +318,60 @@ fn write() {
         // This is a cross section of variables and other kinds of nodes that we want to write to
         let nodes_to_write = vec![
             // 1. a variable value
-            write_value(&node_ids[0], AttributeId::Value, DataValue::new_now(100 as i32)),
+            write_value(
+                &node_ids[0],
+                AttributeId::Value,
+                DataValue::new_now(100 as i32),
+            ),
             // 2. a variable with a bad attribute (IsAbstract doesn't exist on a var)
-            write_value(&node_ids[1], AttributeId::IsAbstract, DataValue::new_now(true)),
+            write_value(
+                &node_ids[1],
+                AttributeId::IsAbstract,
+                DataValue::new_now(true),
+            ),
             // 3. a variable value which has no write access
-            write_value(&node_ids[2], AttributeId::Value, DataValue::new_now(200 as i32)),
+            write_value(
+                &node_ids[2],
+                AttributeId::Value,
+                DataValue::new_now(200 as i32),
+            ),
             // 4. a node of some kind other than variable
-            write_value(&ReferenceTypeId::HasEncoding.into(), AttributeId::IsAbstract, DataValue::new_now(false)),
+            write_value(
+                &ReferenceTypeId::HasEncoding.into(),
+                AttributeId::IsAbstract,
+                DataValue::new_now(false),
+            ),
             // 5. a node with some kind other than variable with no write mask
-            write_value(&ReferenceTypeId::HasChild.into(), AttributeId::IsAbstract, DataValue::new_now(false)),
+            write_value(
+                &ReferenceTypeId::HasChild.into(),
+                AttributeId::IsAbstract,
+                DataValue::new_now(false),
+            ),
             // 6. a non existent variable
-            write_value(&NodeId::new(2, "vxxx"), AttributeId::Value, DataValue::new_now(100i32)),
+            write_value(
+                &NodeId::new(2, "vxxx"),
+                AttributeId::Value,
+                DataValue::new_now(100i32),
+            ),
             // 7. wrong type for attribute
-            write_value(&node_ids[6], AttributeId::AccessLevel, DataValue::new_now(-1i8)),
+            write_value(
+                &node_ids[6],
+                AttributeId::AccessLevel,
+                DataValue::new_now(-1i8),
+            ),
             // 8. a data value with no value
             write_value(&node_ids[7], AttributeId::Value, data_value_empty),
         ];
 
         let nodes_to_write_len = nodes_to_write.len();
 
-        let response = write_request(server_state, session, address_space.clone(), ats, nodes_to_write);
+        let response = write_request(
+            server_state,
+            session,
+            address_space.clone(),
+            ats,
+            nodes_to_write,
+        );
         let results = response.results.unwrap();
         assert_eq!(results.len(), nodes_to_write_len);
 
@@ -300,7 +406,7 @@ fn write_bytestring_to_byte_array() {
         // Create a variable that is an array of bytes
         let node_id = NodeId::next_numeric(2);
         {
-            let mut address_space = trace_write_lock_unwrap!(address_space);
+            let mut address_space = trace_write_lock!(address_space);
             let _ = VariableBuilder::new(&node_id, var_name(0), "")
                 .data_type(DataTypeId::Byte)
                 .value_rank(1)
@@ -311,30 +417,36 @@ fn write_bytestring_to_byte_array() {
         }
 
         let bytes = ByteString::from(vec![0x1u8, 0x2u8, 0x3u8, 0x4u8]);
-        let nodes_to_write = vec![
-            write_value(&node_id, AttributeId::Value, DataValue::new_now(bytes)),
-        ];
+        let nodes_to_write = vec![write_value(
+            &node_id,
+            AttributeId::Value,
+            DataValue::new_now(bytes),
+        )];
 
         // Do a write
-        let response = write_request(server_state, session, address_space.clone(), ats, nodes_to_write);
+        let response = write_request(
+            server_state,
+            session,
+            address_space.clone(),
+            ats,
+            nodes_to_write,
+        );
         let results = response.results.unwrap();
 
         // Expect the write to have succeeded
         assert_eq!(results[0], StatusCode::Good);
 
         // Test the node expecting it to be an array with 4 Byte values
-        validate_variable_value(address_space, &node_id, |value| {
-            match value {
-                Variant::Array(array) => {
-                    let values = &array.values;
-                    assert_eq!(values.len(), 4);
-                    assert_eq!(values[0], Variant::Byte(0x1u8));
-                    assert_eq!(values[1], Variant::Byte(0x2u8));
-                    assert_eq!(values[2], Variant::Byte(0x3u8));
-                    assert_eq!(values[3], Variant::Byte(0x4u8));
-                }
-                _ => panic!()
+        validate_variable_value(address_space, &node_id, |value| match value {
+            Variant::Array(array) => {
+                let values = &array.values;
+                assert_eq!(values.len(), 4);
+                assert_eq!(values[0], Variant::Byte(0x1u8));
+                assert_eq!(values[1], Variant::Byte(0x2u8));
+                assert_eq!(values[2], Variant::Byte(0x3u8));
+                assert_eq!(values[3], Variant::Byte(0x4u8));
             }
+            _ => panic!(),
         });
     });
 }
@@ -347,32 +459,53 @@ fn write_index_range() {
         let node_id_1 = NodeId::next_numeric(2);
         let node_id_2 = NodeId::next_numeric(2);
 
-        [&node_id_1, &node_id_2].iter().enumerate().for_each(|(i, node_id)| {
-            let mut address_space = trace_write_lock_unwrap!(address_space);
-            let _ = VariableBuilder::new(node_id, var_name(i), "")
-                .data_type(DataTypeId::Byte)
-                .value_rank(1)
-                .value(vec![0u8; 16])
-                .organized_by(ObjectId::RootFolder)
-                .writable()
-                .insert(&mut address_space);
-        });
+        [&node_id_1, &node_id_2]
+            .iter()
+            .enumerate()
+            .for_each(|(i, node_id)| {
+                let mut address_space = trace_write_lock!(address_space);
+                let _ = VariableBuilder::new(node_id, var_name(i), "")
+                    .data_type(DataTypeId::Byte)
+                    .value_rank(1)
+                    .value(vec![0u8; 16])
+                    .organized_by(ObjectId::RootFolder)
+                    .writable()
+                    .insert(&mut address_space);
+            });
 
         let index: usize = 12;
         let index_expected_value = 73u8;
         let index_bytes = Variant::from(vec![index_expected_value]);
 
         let (range_min, range_max) = (4 as usize, 12 as usize);
-        let range_bytes = vec![0x1u8, 0x2u8, 0x3u8, 0x4u8, 0x5u8, 0x6u8, 0x7u8, 0x8u8, 0x9u8];
+        let range_bytes = vec![
+            0x1u8, 0x2u8, 0x3u8, 0x4u8, 0x5u8, 0x6u8, 0x7u8, 0x8u8, 0x9u8,
+        ];
         let range_value = Variant::from(range_bytes.clone());
 
         let nodes_to_write = vec![
-            write_value_index_range(&node_id_1, AttributeId::Value, format!("{}", index), DataValue::new_now(index_bytes)),
-            write_value_index_range(&node_id_2, AttributeId::Value, format!("{}:{}", range_min, range_max), DataValue::new_now(range_value)),
+            write_value_index_range(
+                &node_id_1,
+                AttributeId::Value,
+                format!("{}", index),
+                DataValue::new_now(index_bytes),
+            ),
+            write_value_index_range(
+                &node_id_2,
+                AttributeId::Value,
+                format!("{}:{}", range_min, range_max),
+                DataValue::new_now(range_value),
+            ),
         ];
 
         // Do a write
-        let response = write_request(server_state, session, address_space.clone(), ats, nodes_to_write);
+        let response = write_request(
+            server_state,
+            session,
+            address_space.clone(),
+            ats,
+            nodes_to_write,
+        );
         let results = response.results.unwrap();
 
         // Expect the write to have succeeded
@@ -386,11 +519,15 @@ fn write_index_range() {
                     assert_eq!(values.len(), 16);
                     values.iter().enumerate().for_each(|(i, v)| {
                         // Only one element set, others should not be set
-                        let expected = if i == index { index_expected_value } else { 0u8 };
+                        let expected = if i == index {
+                            index_expected_value
+                        } else {
+                            0u8
+                        };
                         assert_eq!(*v, Variant::Byte(expected));
                     });
                 }
-                _ => panic!()
+                _ => panic!(),
             }
         });
 
@@ -409,7 +546,7 @@ fn write_index_range() {
                         assert_eq!(*v, Variant::Byte(expected));
                     });
                 }
-                _ => panic!()
+                _ => panic!(),
             }
         });
     });
@@ -417,16 +554,26 @@ fn write_index_range() {
 
 // #[test] fn write_null_value() { /* Write an empty variant to a value and see that it is allowed */}
 
-
 struct DataProvider;
 
 impl HistoricalDataProvider for DataProvider {
-    fn read_raw_modified_details(&self, _address_space: Arc<RwLock<AddressSpace>>, _request: ReadRawModifiedDetails, _timestamps_to_return: TimestampsToReturn, _release_continuation_points: bool, _nodes_to_read: &[HistoryReadValueId]) -> Result<Vec<HistoryReadResult>, StatusCode> {
+    fn read_raw_modified_details(
+        &self,
+        _address_space: Arc<RwLock<AddressSpace>>,
+        _request: ReadRawModifiedDetails,
+        _timestamps_to_return: TimestampsToReturn,
+        _release_continuation_points: bool,
+        _nodes_to_read: &[HistoryReadValueId],
+    ) -> Result<Vec<HistoryReadResult>, StatusCode> {
         info!("DataProvider's read_raw_modified_details");
         Ok(DataProvider::historical_read_result())
     }
 
-    fn delete_raw_modified_details(&self, _address_space: Arc<RwLock<AddressSpace>>, _request: DeleteRawModifiedDetails) -> Result<Vec<StatusCode>, StatusCode> {
+    fn delete_raw_modified_details(
+        &self,
+        _address_space: Arc<RwLock<AddressSpace>>,
+        _request: DeleteRawModifiedDetails,
+    ) -> Result<Vec<StatusCode>, StatusCode> {
         info!("DataProvider's delete_raw_modified_details");
         Ok(vec![StatusCode::Good])
     }
@@ -434,24 +581,21 @@ impl HistoricalDataProvider for DataProvider {
 
 impl DataProvider {
     pub fn historical_read_result() -> Vec<HistoryReadResult> {
-        vec![
-            HistoryReadResult {
-                status_code: StatusCode::Good,
-                continuation_point: ByteString::null(),
-                history_data: ExtensionObject::null(),
-            }]
+        vec![HistoryReadResult {
+            status_code: StatusCode::Good,
+            continuation_point: ByteString::null(),
+            history_data: ExtensionObject::null(),
+        }]
     }
 }
 
 fn nodes_to_read() -> Vec<HistoryReadValueId> {
-    vec![
-        HistoryReadValueId {
-            node_id: NodeId::new(2, "test"),
-            index_range: UAString::null(),
-            data_encoding: QualifiedName::null(), // TODO
-            continuation_point: ByteString::null(),
-        }
-    ]
+    vec![HistoryReadValueId {
+        node_id: NodeId::new(2, "test"),
+        index_range: UAString::null(),
+        data_encoding: QualifiedName::null(), // TODO
+        continuation_point: ByteString::null(),
+    }]
 }
 
 fn read_raw_modified_details() -> ReadRawModifiedDetails {
@@ -475,7 +619,10 @@ fn history_read_nothing_to_do_1() {
         // Register a history data provider
         // Send a valid read details command but with no nodes to read
         let read_raw_modified_details = read_raw_modified_details();
-        let history_read_details = ExtensionObject::from_encodable(ObjectId::ReadRawModifiedDetails_Encoding_DefaultBinary, &read_raw_modified_details);
+        let history_read_details = ExtensionObject::from_encodable(
+            ObjectId::ReadRawModifiedDetails_Encoding_DefaultBinary,
+            &read_raw_modified_details,
+        );
         let request = HistoryReadRequest {
             request_header: make_request_header(),
             history_read_details,
@@ -483,8 +630,14 @@ fn history_read_nothing_to_do_1() {
             release_continuation_points: true,
             nodes_to_read: None,
         };
-        let response: ServiceFault = supported_message_as!(ats.history_read(server_state, session, address_space.clone(), &request), ServiceFault);
-        assert_eq!(response.response_header.service_result, StatusCode::BadNothingToDo);
+        let response: ServiceFault = supported_message_as!(
+            ats.history_read(server_state, session, address_space.clone(), &request),
+            ServiceFault
+        );
+        assert_eq!(
+            response.response_header.service_result,
+            StatusCode::BadNothingToDo
+        );
     });
 }
 
@@ -499,8 +652,14 @@ fn history_read_nothing_history_operation_invalid() {
             release_continuation_points: true,
             nodes_to_read: Some(nodes_to_read()),
         };
-        let response: ServiceFault = supported_message_as!(ats.history_read(server_state, session, address_space, &request), ServiceFault);
-        assert_eq!(response.response_header.service_result, StatusCode::BadHistoryOperationInvalid);
+        let response: ServiceFault = supported_message_as!(
+            ats.history_read(server_state, session, address_space, &request),
+            ServiceFault
+        );
+        assert_eq!(
+            response.response_header.service_result,
+            StatusCode::BadHistoryOperationInvalid
+        );
     });
 }
 
@@ -515,7 +674,10 @@ fn history_read_nothing_data_provider() {
 
         // Call ReadRawModifiedDetails on the registered callback and expect a call back
         let read_raw_modified_details = read_raw_modified_details();
-        let history_read_details = ExtensionObject::from_encodable(ObjectId::ReadRawModifiedDetails_Encoding_DefaultBinary, &read_raw_modified_details);
+        let history_read_details = ExtensionObject::from_encodable(
+            ObjectId::ReadRawModifiedDetails_Encoding_DefaultBinary,
+            &read_raw_modified_details,
+        );
         let request = HistoryReadRequest {
             request_header: make_request_header(),
             history_read_details,
@@ -523,7 +685,10 @@ fn history_read_nothing_data_provider() {
             release_continuation_points: true,
             nodes_to_read: Some(nodes_to_read()),
         };
-        let response: HistoryReadResponse = supported_message_as!(ats.history_read(server_state, session, address_space, &request), HistoryReadResponse);
+        let response: HistoryReadResponse = supported_message_as!(
+            ats.history_read(server_state, session, address_space, &request),
+            HistoryReadResponse
+        );
         let expected_read_result = DataProvider::historical_read_result();
         assert_eq!(response.results, Some(expected_read_result));
     });
@@ -549,8 +714,14 @@ fn history_update_nothing_to_do_1() {
             request_header: make_request_header(),
             history_update_details: None,
         };
-        let response: ServiceFault = supported_message_as!(ats.history_update(server_state, session, address_space, &request), ServiceFault);
-        assert_eq!(response.response_header.service_result, StatusCode::BadNothingToDo);
+        let response: ServiceFault = supported_message_as!(
+            ats.history_update(server_state, session, address_space, &request),
+            ServiceFault
+        );
+        assert_eq!(
+            response.response_header.service_result,
+            StatusCode::BadNothingToDo
+        );
     });
 }
 
@@ -562,8 +733,14 @@ fn history_update_nothing_to_do_2() {
             request_header: make_request_header(),
             history_update_details: Some(vec![]),
         };
-        let response: ServiceFault = supported_message_as!(ats.history_update(server_state, session, address_space, &request), ServiceFault);
-        assert_eq!(response.response_header.service_result, StatusCode::BadNothingToDo);
+        let response: ServiceFault = supported_message_as!(
+            ats.history_update(server_state, session, address_space, &request),
+            ServiceFault
+        );
+        assert_eq!(
+            response.response_header.service_result,
+            StatusCode::BadNothingToDo
+        );
     });
 }
 
@@ -575,7 +752,10 @@ fn history_update_history_operation_invalid() {
             request_header: make_request_header(),
             history_update_details: Some(vec![ExtensionObject::null()]),
         };
-        let response: HistoryUpdateResponse = supported_message_as!(ats.history_update(server_state, session, address_space, &request), HistoryUpdateResponse);
+        let response: HistoryUpdateResponse = supported_message_as!(
+            ats.history_update(server_state, session, address_space, &request),
+            HistoryUpdateResponse
+        );
         let results = response.results.unwrap();
         assert_eq!(results.len(), 1);
 
@@ -591,17 +771,26 @@ fn history_update_history_operation_unsupported() {
         let delete_raw_modified_details = delete_raw_modified_details();
 
         // Unsupported operation (everything by default)
-        let history_update_details = ExtensionObject::from_encodable(ObjectId::DeleteRawModifiedDetails_Encoding_DefaultBinary, &delete_raw_modified_details);
+        let history_update_details = ExtensionObject::from_encodable(
+            ObjectId::DeleteRawModifiedDetails_Encoding_DefaultBinary,
+            &delete_raw_modified_details,
+        );
         let request = HistoryUpdateRequest {
             request_header: make_request_header(),
             history_update_details: Some(vec![history_update_details]),
         };
-        let response: HistoryUpdateResponse = supported_message_as!(ats.history_update(server_state, session, address_space, &request), HistoryUpdateResponse);
+        let response: HistoryUpdateResponse = supported_message_as!(
+            ats.history_update(server_state, session, address_space, &request),
+            HistoryUpdateResponse
+        );
         let results = response.results.unwrap();
         assert_eq!(results.len(), 1);
 
         let result1 = &results[0];
-        assert_eq!(result1.status_code, StatusCode::BadHistoryOperationUnsupported);
+        assert_eq!(
+            result1.status_code,
+            StatusCode::BadHistoryOperationUnsupported
+        );
     });
 }
 
@@ -618,16 +807,22 @@ fn history_update_data_provider() {
         let delete_raw_modified_details = delete_raw_modified_details();
 
         // Supported operation
-        let history_update_details = ExtensionObject::from_encodable(ObjectId::DeleteRawModifiedDetails_Encoding_DefaultBinary, &delete_raw_modified_details);
+        let history_update_details = ExtensionObject::from_encodable(
+            ObjectId::DeleteRawModifiedDetails_Encoding_DefaultBinary,
+            &delete_raw_modified_details,
+        );
         let request = HistoryUpdateRequest {
             request_header: make_request_header(),
             history_update_details: Some(vec![history_update_details]),
         };
-        let response: HistoryUpdateResponse = supported_message_as!(ats.history_update(server_state, session, address_space, &request), HistoryUpdateResponse);
+        let response: HistoryUpdateResponse = supported_message_as!(
+            ats.history_update(server_state, session, address_space, &request),
+            HistoryUpdateResponse
+        );
         let results = response.results.unwrap();
         assert_eq!(results.len(), 1);
 
         let result1 = &results[0];
         assert_eq!(result1.status_code, StatusCode::Good);
     });
-}
\ No newline at end of file
+}
